@@ -19,8 +19,9 @@ class Core():
     basic_auth_callback: Callable[[str, str], bool] = None
     enc_dec_jwt_callback: dict = None
     get_user_roles_callback: list = None
+    personal_credentials: tuple[str, str] = None
 
-    def enc_dec_jwt_config(self, func: Callable[[None], dict]) -> None:
+    def enc_dec_jwt_config(self, func: Callable[[None], dict]) -> Callable[[None], dict]:
         """Decorator to verify the JWT token
         :param f: function to be decorated
         :return: the function to wrap should return a dictionary with the following keys:
@@ -29,6 +30,23 @@ class Core():
         self.enc_dec_jwt_callback = func()
         return func
 
+    def personal_credentials_field(self, func: Callable[[None], tuple[str, str]]) -> Callable[[None], tuple[str, str]]:
+        """
+        Decorator to set the personal credentials, if youu dont want to use username and password inside the token
+        then with this you can return a tuple in which the first element is the username and the second is the password
+        but as you want to name that respective fields so the library will validate using the fields you set
+        :param func: function to be decorated
+        :return: the tuple with the username and password with personal names
+
+        :Example:
+        @dec_jwt.personal_credentials_field
+        
+        def get_personal_credentials():
+            return "my_username_personal_name_field", "my_password_personal_name_field"
+        """
+        self.personal_credentials = func()
+        return func
+
     def verify_dict_config(self, config: str) -> None:
         """Method that veryfies the JWT configuration generator and for basic auth
         :param config: str to identify which configuration to verify"""
@@ -69,7 +87,7 @@ class Core():
         :param status_code: status code in int format"""
         abort(make_response(jsonify({"error": error}), status_code))
 
-    def ensure_sync(self, func) -> Callable:
+    def ensure_sync(self, func: Callable) -> Callable:
         """Decorator to ensure the function is synchronous
         :param f: function to be decorated
         :return: the function to wrap"""
@@ -89,12 +107,17 @@ class GenJwt(Core):
         """
         if not self.jwt_fields_attr:
             self.gen_abort_error("jwt_claims decorator and function is not defined", 500)
+        if self.personal_credentials is not None:
+            bauth_credentials[self.personal_credentials[0]] = bauth_credentials["username"]
+            bauth_credentials[self.personal_credentials[1]] = bauth_credentials["password"]
+            del bauth_credentials["username"]
+            del bauth_credentials["password"]
         payload = bauth_credentials
         payload.update(self.jwt_fields_attr)
             
         return payload
     
-    def __dec_set_basic_auth(self) -> None:
+    def __dec_set_basic_auth(self) -> Optional[bool]:
         """
         Method to decode and verify the basic auth credentials in the expected format
         """
@@ -119,7 +142,7 @@ class GenJwt(Core):
         else:
             self.gen_abort_error("basic_auth decorator and function is not defined", 500)
     
-    def __encode_jwt(self, payload) -> Optional[str]:
+    def __encode_jwt(self, payload: dict) -> Optional[str]:
         """
         Method to encode the JWT token using the key and algorithm specified in the enc_dec_jwt_config decorator
         that returns the dictionary with the configuration.
@@ -159,6 +182,9 @@ class GenJwt(Core):
         return func
 
     def generate_jwt(self, func=None, roles=None):
+        """
+        Decorator to generate the JWT token through the function of the endpoint that responds the token
+        """
         if func is not None and (roles is not None):
             raise ValueError("role and optional are the only supported arguments")
         def func_to_receive(func):
@@ -188,7 +214,7 @@ class DecJwt(Core):
         self.credentials_success_callback: bool = None
         self.get_jwt_claims_to_verify_callback: list[str] = None
     
-    def __decode_jwt(self) -> Optional[str]:
+    def __decode_jwt(self) -> Optional[dict]:
         """
         Decode the JWT token using the key and algorithm specified in the enc_dec_jwt_config decorator
         that returns the dictionary with the configuration.
@@ -208,7 +234,7 @@ class DecJwt(Core):
             decoded_token = None
         return decoded_token
 
-    def __verify_token(self, token) -> None:
+    def __verify_token(self, token: dict) -> None:
         """Verify the token, if its None the something went wrong with the decoding of the token.
         If the token is not None, then verify the claims if you implement the get_jwt_claims_to_verify decorator.
         By default the method verify if there is at least one claim inside jwt, if not then invalid token error will appear.
@@ -223,22 +249,32 @@ class DecJwt(Core):
                         self.gen_abort_error(f"The claim {claim} is not in the token", 400)
             if len(token) < 1:
                 self.gen_abort_error("Invalid token", 401)
-            if ("username" not in token) or ("password" not in token):
-                self.gen_abort_error("Invalid token", 401)
+            if self.personal_credentials is not None:
+                per_username = self.personal_credentials[0]
+                per_password = self.personal_credentials[1]
+                if (per_username not in token) or (per_password not in token):
+                    self.gen_abort_error("Invalid token", 401)
+            else:
+                if ("username" not in token) or ("password" not in token):
+                    self.gen_abort_error("Invalid token", 401)
             keys_to_validate = self.get_jwt_claims_to_verify_callback
             for key in keys_to_validate:
                 if key not in token:
                     self.gen_abort_error("Credentials to validate for authentication inside token are not correct", 401)
     
-    def __authenticate_credentials(self, token) -> None:
+    def __authenticate_credentials(self, token: dict) -> bool:
         """
         Verify the credentials of the user, if the credentials are not correct then the user will be unauthorized
         :param token: token to verify the credentials
         """
         if self.credentials_success_callback is None:
             self.gen_abort_error("get_credentials_success decorator is not set", 500)
-        username_jwt = token["username"]
-        password_jwt = token["password"]
+        if self.personal_credentials is None:
+            username_jwt = token["username"]
+            password_jwt = token["password"]
+        else:
+            username_jwt = token[self.personal_credentials[0]]
+            password_jwt = token[self.personal_credentials[1]]
         return self.ensure_sync(self.credentials_success_callback)(username_jwt, password_jwt)
 
     def __set_token_as_attr(self, token: dict) -> None:
@@ -255,7 +291,7 @@ class DecJwt(Core):
         :return: the function to wrap that returns the a boolean field"""
         self.get_jwt_claims_to_verify_callback = func()
 
-    def verify_jwt_credentials(self, func) -> Callable[[str, str], dict]:
+    def verify_jwt_credentials(self, func: Callable[[str, str], bool]) -> Callable[[str, str], bool]:
         """Decorator to get the credentials from database or whatever part
         to verify the token fields later
         :param func: function to be decorated
@@ -265,6 +301,10 @@ class DecJwt(Core):
         return func
     
     def login_required(self, func=None, roles=None):
+        """
+        Decorator to verify the JWT token through the function of the endpoints that
+        are requested by the user, also validates the roles setted in the endpoint.
+        """
         if func is not None and (roles is not None):
             raise ValueError("role and optional are the only supported arguments")
         def func_to_receive(func):
@@ -275,11 +315,14 @@ class DecJwt(Core):
                 else:
                     token = self.__decode_jwt()
                     self.__verify_token(token)
-                    self.verify_user_roles(roles, token["username"])
 
                     grant_access = self.__authenticate_credentials(token)
                     if not grant_access:
                         self.gen_abort_error("The credentials are not correct", 401)
+                    if self.personal_credentials is not None:
+                        self.verify_user_roles(roles, token[self.personal_credentials[0]])
+                    else:
+                        self.verify_user_roles(roles, token["username"])
                     self.__set_token_as_attr(token)
 
                 return self.ensure_sync(func)(*args, **kwargs)
